﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace GeoSerializerTest
{

    

    public partial class Form1 : Form
    {
        // file streams for the serialization/deserialization
        private FileStream output;  // Stream for writing to a file.
        private FileStream input;  // Stream for reading from a file.

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.CurrentExtent = ExtentHelper.GetDrawingExtent(new RectangleShape(-180, 90, 180, -90), winformsMap1.Width, winformsMap1.Height);

            // Setup the culture overlay
            InMemoryFeatureLayer cultureLayer = new InMemoryFeatureLayer();
            cultureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(100, GeoColor.SimpleColors.Green);
            cultureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay cultureOverlay = new LayerOverlay();
            cultureOverlay.Layers.Add("CultureLayer", cultureLayer);
            cultureOverlay.Layers["CultureLayer"].Name = "CultureLayer";
            winformsMap1.Overlays.Add("CultureOverlay", cultureOverlay);
            winformsMap1.Overlays["CultureOverlay"].Name = "CultureOverlay";

            winformsMap1.Refresh();
        }

        private void Save_Click(object sender, EventArgs e)
        {
            output = new FileStream(Application.StartupPath + @"\SaveFile.txt", FileMode.Create, FileAccess.Write);
            using (output)
            {
                SerializationFunctions.WriteSerializedOverlays(output, winformsMap1.Overlays);
            }

            winformsMap1.Overlays.Clear();
            winformsMap1.Refresh();
        }

        private void Load_Click(object sender, EventArgs e)
        {
            input = new FileStream(Application.StartupPath + @"\SaveFile.txt", FileMode.Open, FileAccess.Read);
            using (input)
            {
                Collection<Overlay> overlays = SerializationFunctions.ReadSerializedOverlays(input);
                foreach (var overlay in overlays)
                {
                    LayerOverlay layerOverlay = (LayerOverlay)overlay;
                    foreach (Layer layer in layerOverlay.Layers)
                    {
                        layer.Close();
                        layer.Open();
                    }

                    winformsMap1.Overlays.Add(overlay.Name, overlay);
                }
            }

            winformsMap1.Refresh();
        }

        public static void AddShapeFileFeatureLayers(IEnumerable<string> fileNames, LayerOverlay layerOverlay, WinformsMap winformsMap)
        {
            foreach (string fileName in fileNames)
            {
                try
                {
                    string layerName = fileName;
                    if (!layerOverlay.Layers.Contains(layerName))
                    {
                        string indexFile = Path.ChangeExtension(fileName, ".idx");
                        ShapeFileFeatureLayer.BuildIndexFile(fileName, BuildIndexMode.DoNotRebuild);

                        ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(fileName);
                        layer.Encoding = Encoding.ASCII;
                        layer.Name = layerName;
                        ExplorerHelper.SetStyleByWellKnownType(layer);
                        layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                        string prjFileName = fileName.Substring(0, fileName.Length - 4) + ".prj";
                        if (File.Exists(prjFileName))
                        {
                            Proj4Projection proj4 = new Proj4Projection();

                            string prjFileText = System.IO.File.ReadAllText(prjFileName);

                            proj4.InternalProjectionParametersString = Proj4Projection.ConvertPrjToProj4(prjFileText);
                            proj4.ExternalProjectionParametersString = Proj4Projection.GetWgs84ParametersString();  //WGS84 (Geodetic) 
                            layer.FeatureSource.Projection = proj4;
                        }

                        try
                        {
                            layer.Open();
                            layer.Validate();
                            layer.Close();

                            layerOverlay.Layers.Add(layerName, layer);
                            Debug.Print("LayerOverlay Name: " + layerOverlay.Name);
                        }
                        finally
                        {
                        }
                    }
                }
                catch (FileLoadException fle)
                {
                    MessageBox.Show(fle.Message, fle.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
                catch (InvalidOperationException ex)
                {
                    MessageBox.Show(ex.Message, ex.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, ex.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
            }
        }

        private void btnAddShape_Click(object sender, EventArgs e)
        {


            openFileDialog.Filter = "Shape Files (*.shp)|*.shp";
            openFileDialog.Title = "Open Shape Files";
            openFileDialog.Multiselect = true;

            LayerOverlay layerOverlay = ((LayerOverlay)winformsMap1.Overlays["CultureOverlay"]);

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {

                Collection<string> featureFilenames = new Collection<string>();

                foreach (string filename in openFileDialog.FileNames)
                {
                    featureFilenames.Add(filename);
                }

                AddShapeFileFeatureLayers(featureFilenames, layerOverlay, winformsMap1);

                winformsMap1.Refresh();

            }
        }
    }
}
