﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Runtime.Serialization;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace GeoSerializerTest
{
    class SerializationFunctions
    {
        public static void WriteSerializedOverlays(FileStream output, Collection<Overlay> overlays)
        {
            GeoSerializer serializer = new GeoSerializer();
            OverlaysSerializer overlayRecord = new OverlaysSerializer();
            foreach (var overlay in overlays)
            {
                overlayRecord.FullOverlays.Add(overlay);
            }

            try
            {
                serializer.Serialize(overlayRecord, output);
            }
            catch (ArgumentException argEx)
            {
                string errors = string.Format("Error Writing to File in Overlay Layer: {0}\n\n{1}\n\n{2}\n\n",
                    "Overlays", argEx.Message, argEx.StackTrace);

                MessageBox.Show(errors, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                FileStream errorFile = new FileStream(Application.StartupPath + @"\Debug_Error.txt", FileMode.Append, FileAccess.Write);
                errorFile.Write(ASCIIEncoding.ASCII.GetBytes(errors), 0, ASCIIEncoding.ASCII.GetByteCount(errors));
                errorFile.Close();
            }
            catch (SerializationException)
            {
                MessageBox.Show("Error Writing to File", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            // notify user if error occurs regarding parameter format
            catch (FormatException)
            {
                MessageBox.Show("Invalid Format", "Error",
                   MessageBoxButtons.OK, MessageBoxIcon.Error);
            } // end catch
        }

        public static Collection<Overlay> ReadSerializedOverlays(FileStream input)
        {
            GeoSerializer reader = new GeoSerializer();
            Collection<Overlay> overlays = null;
            try
            {
                OverlaysSerializer overlayRecord = (OverlaysSerializer)reader.Deserialize(input);
                overlays = overlayRecord.FullOverlays;
            }
            catch (SerializationException)
            {
                input.Close();
                // notify user if no RecordSerializables in file
                MessageBox.Show("No more records in file", string.Empty,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (InvalidCastException)
            {
                //Do nothing, carry on.
            }
            catch (ArgumentException)
            {
                //Do nothing, carry on.
            }

            return overlays;
        }
    }
}
